@@ -30,8 +30,8 @@
 #include <locale.h>
 #include <errno.h>
 
-#ifdef HAVE_SYS_SIGNAL_H
-# include <sys/signal.h>
+#ifdef HAVE_SIGNAL_H
+# include <signal.h>
 #endif
 
 #ifdef ENABLE_NLS
@@ -76,6 +76,10 @@
 # include "location-gnome-clock.h"
 #endif
 
+#ifdef ENABLE_GEOCLUE
+# include "location-geoclue.h"
+#endif
+
 
 /* Union of state data for gamma adjustment methods */
 typedef union {
@@ -139,11 +143,28 @@ typedef union {
 #ifdef ENABLE_GNOME_CLOCK
 	location_gnome_clock_state_t gnome_clock;
 #endif
+#ifdef ENABLE_GEOCLUE
+	location_geoclue_state_t geoclue;
+#endif
 } location_state_t;
 
 
 /* Location provider method structs */
 static const location_provider_t location_providers[] = {
+#ifdef ENABLE_GEOCLUE
+	{
+		"geoclue",
+		(location_provider_init_func *)location_geoclue_init,
+		(location_provider_start_func *)location_geoclue_start,
+		(location_provider_free_func *)location_geoclue_free,
+		(location_provider_print_help_func *)
+		location_geoclue_print_help,
+		(location_provider_set_option_func *)
+		location_geoclue_set_option,
+		(location_provider_get_location_func *)
+		location_geoclue_get_location
+	},
+#endif
 #ifdef ENABLE_GNOME_CLOCK
 	{
 		"gnome-clock",
@@ -180,12 +201,15 @@ static const location_provider_t location_providers[] = {
 #define MAX_LON   180.0
 #define MIN_TEMP   1000
 #define MAX_TEMP  10000
+#define MIN_BRIGHTNESS  0.1
+#define MAX_BRIGHTNESS  1.0
 #define MIN_GAMMA   0.1
 #define MAX_GAMMA  10.0
 
 /* Default values for parameters. */
 #define DEFAULT_DAY_TEMP    5500
 #define DEFAULT_NIGHT_TEMP  3700
+#define DEFAULT_BRIGHTNESS   1.0
 #define DEFAULT_GAMMA        1.0
 
 /* The color temperature when no adjustment is applied. */
@@ -202,11 +226,12 @@ static const location_provider_t location_providers[] = {
 typedef enum {
 	PROGRAM_MODE_CONTINUAL,
 	PROGRAM_MODE_ONE_SHOT,
-	PROGRAM_MODE_RESET
+	PROGRAM_MODE_RESET,
+	PROGRAM_MODE_MANUAL
 } program_mode_t;
 
 
-#ifdef HAVE_SYS_SIGNAL_H
+#ifdef HAVE_SIGNAL_H
 
 static volatile sig_atomic_t exiting = 0;
 static volatile sig_atomic_t disable = 0;
@@ -225,12 +250,12 @@ sigdisable(int signo)
 	disable = 1;
 }
 
-#else /* ! HAVE_SYS_SIGNAL_H */
+#else /* ! HAVE_SIGNAL_H */
 
 static int exiting = 0;
 static int disable = 0;
 
-#endif /* ! HAVE_SYS_SIGNAL_H */
+#endif /* ! HAVE_SIGNAL_H */
 
 
 /* Calculate color temperature for the specified solar elevation. */
@@ -280,13 +305,15 @@ print_help(const char *program_name)
 	/* TRANSLATORS: help output 3
 	   no-wrap */
 	fputs(_("  -h\t\tDisplay this help message\n"
-		"  -v\t\tVerbose output\n"), stdout);
-	fputs("\n", stdout);	
+		"  -v\t\tVerbose output\n"
+                "  -V\t\tShow program version\n"), stdout);
+	fputs("\n", stdout);
 
 	/* TRANSLATORS: help output 4
 	   `list' must not be translated
 	   no-wrap */
-	fputs(_("  -c FILE\tLoad settings from specified configuration file\n"
+	fputs(_("  -b N\t\tScreen brightness to apply (max is 1.0)\n"
+                "  -c FILE\tLoad settings from specified configuration file\n"
 		"  -g R:G:B\tAdditional gamma correction to apply\n"
 		"  -l LAT:LON\tYour current location\n"
 		"  -l PROVIDER\tSelect provider for automatic"
@@ -296,6 +323,7 @@ print_help(const char *program_name)
 		"  \t\t(Type `list' to see available methods)\n"
 		"  -o\t\tOne shot mode (do not continously adjust"
 		" color temperature)\n"
+		"  -O TEMP\tOne shot manual mode (set color temperature)\n"
 		"  -x\t\tReset mode (remove adjustment from screen)\n"
 		"  -r\t\tDisable temperature transitions\n"
 		"  -t DAY:NIGHT\tColor temperature to set at daytime/night\n"),
@@ -533,6 +561,8 @@ parse_gamma_string(const char *str, float gamma[])
 		gamma[1] = atof(g_s); /* Blue */
 		gamma[2] = atof(s); /* Green */
 	}
+
+	return 0;
 }
 
 static const gamma_method_t *
@@ -584,9 +614,11 @@ main(int argc, char *argv[])
 	/* Initialize settings to NULL values. */
 	char *config_filepath = NULL;
 
+	int temp_set = -1;
 	int temp_day = -1;
 	int temp_night = -1;
 	float gamma[3] = { NAN, NAN, NAN };
+	float brightness = NAN;
 
 	const gamma_method_t *method = NULL;
 	char *method_args = NULL;
@@ -601,8 +633,11 @@ main(int argc, char *argv[])
 
 	/* Parse command line arguments. */
 	int opt;
-	while ((opt = getopt(argc, argv, "c:g:hl:m:ort:vx")) != -1) {
+	while ((opt = getopt(argc, argv, "b:c:g:hl:m:oO:rt:vVx")) != -1) {
 		switch (opt) {
+		case 'b':
+			brightness = atof(optarg);
+			break;
 		case 'c':
 			if (config_filepath != NULL) free(config_filepath);
 			config_filepath = strdup(optarg);
@@ -699,6 +734,10 @@ main(int argc, char *argv[])
 		case 'o':
 			mode = PROGRAM_MODE_ONE_SHOT;
 			break;
+		case 'O':
+			mode = PROGRAM_MODE_MANUAL;
+			temp_set = atoi(optarg);
+			break;
 		case 'r':
 			transition = 0;
 			break;
@@ -718,6 +757,10 @@ main(int argc, char *argv[])
 		case 'v':
 			verbose = 1;
 			break;
+                case 'V':
+                        printf("%s\n", PACKAGE_STRING);
+                        exit(EXIT_SUCCESS);
+                        break;
 		case 'x':
 			mode = PROGRAM_MODE_RESET;
 			break;
@@ -758,6 +801,11 @@ main(int argc, char *argv[])
 				if (transition < 0) {
 					transition = !!atoi(setting->value);
 				}
+			} else if (strcasecmp(setting->name,
+					      "brightness") == 0) {
+				if (isnan(brightness)) {
+					brightness = atof(setting->value);
+				}
 			} else if (strcasecmp(setting->name, "gamma") == 0) {
 				if (isnan(gamma[0])) {
 					r = parse_gamma_string(setting->value,
@@ -810,15 +858,20 @@ main(int argc, char *argv[])
 	   the config file nor on the command line. */
 	if (temp_day < 0) temp_day = DEFAULT_DAY_TEMP;
 	if (temp_night < 0) temp_night = DEFAULT_NIGHT_TEMP;
+	if (isnan(brightness)) brightness = DEFAULT_BRIGHTNESS;
 	if (isnan(gamma[0])) gamma[0] = gamma[1] = gamma[2] = DEFAULT_GAMMA;
 	if (transition < 0) transition = 1;
 
+	float lat = NAN;
+	float lon = NAN;
+
 	/* Initialize location provider. If provider is NULL
 	   try all providers until one that works is found. */
 	location_state_t location_state;
 
-	/* Location is not needed for reset mode. */
-	if (mode != PROGRAM_MODE_RESET) {
+	/* Location is not needed for reset mode
+	   or for manual temperature setting. */
+	if (mode != PROGRAM_MODE_RESET && mode != PROGRAM_MODE_MANUAL) {
 		if (provider != NULL) {
 			/* Use provider specified on command line. */
 			r = provider_try_start(provider, &location_state,
@@ -852,60 +905,77 @@ main(int argc, char *argv[])
 				exit(EXIT_FAILURE);
 			}
 		}
-	}
 
-	float lat = NAN;
-	float lon = NAN;
-
-	if (mode != PROGRAM_MODE_RESET) {
 		/* Get current location. */
 		r = provider->get_location(&location_state, &lat, &lon);
 		if (r < 0) {
-			fputs(_("Unable to get location from provider.\n"),
-			      stderr);
-			exit(EXIT_FAILURE);
+		        fputs(_("Unable to get location from provider.\n"),
+		              stderr);
+		        exit(EXIT_FAILURE);
 		}
-
+	
 		provider->free(&location_state);
-
+	
 		if (verbose) {
-			/* TRANSLATORS: Append degree symbols if possible. */
-			printf(_("Location: %f, %f\n"), lat, lon);
+		        /* TRANSLATORS: Append degree symbols if possible. */
+		        printf(_("Location: %f, %f\n"), lat, lon);
 		}
-
+	
 		/* Latitude */
 		if (lat < MIN_LAT || lat > MAX_LAT) {
-			/* TRANSLATORS: Append degree symbols if possible. */
-			fprintf(stderr,
-				_("Latitude must be between %.1f and %.1f.\n"),
-				MIN_LAT, MAX_LAT);
-			exit(EXIT_FAILURE);
+		        /* TRANSLATORS: Append degree symbols if possible. */
+		        fprintf(stderr,
+		                _("Latitude must be between %.1f and %.1f.\n"),
+		                MIN_LAT, MAX_LAT);
+		        exit(EXIT_FAILURE);
 		}
-
+	
 		/* Longitude */
 		if (lon < MIN_LON || lon > MAX_LON) {
-			/* TRANSLATORS: Append degree symbols if possible. */
+		        /* TRANSLATORS: Append degree symbols if possible. */
+		        fprintf(stderr,
+		                _("Longitude must be between"
+		                  " %.1f and %.1f.\n"), MIN_LON, MAX_LON);
+		        exit(EXIT_FAILURE);
+		}
+
+		/* Color temperature at daytime */
+		if (temp_day < MIN_TEMP || temp_day >= MAX_TEMP) {
 			fprintf(stderr,
-				_("Longitude must be between"
-				  " %.1f and %.1f.\n"), MIN_LON, MAX_LON);
+				_("Temperature must be between %uK and %uK.\n"),
+				MIN_TEMP, MAX_TEMP);
+			exit(EXIT_FAILURE);
+		}
+	
+		/* Color temperature at night */
+		if (temp_night < MIN_TEMP || temp_night >= MAX_TEMP) {
+			fprintf(stderr,
+				_("Temperature must be between %uK and %uK.\n"),
+				MIN_TEMP, MAX_TEMP);
 			exit(EXIT_FAILURE);
 		}
 	}
 
-	/* Color temperature at daytime */
-	if (temp_day < MIN_TEMP || temp_day >= MAX_TEMP) {
+	if (mode == PROGRAM_MODE_MANUAL) {
+		/* Check color temperature to be set */
+		if (temp_set < MIN_TEMP || temp_set >= MAX_TEMP) {
+			fprintf(stderr,
+				_("Temperature must be between %uK and %uK.\n"),
+				MIN_TEMP, MAX_TEMP);
+			exit(EXIT_FAILURE);
+		}
+	}
+
+	/* Brightness */
+	if (brightness < MIN_BRIGHTNESS || brightness > MAX_BRIGHTNESS) {
 		fprintf(stderr,
-			_("Temperature must be between %uK and %uK.\n"),
-			MIN_TEMP, MAX_TEMP);
+			_("Brightness value must be between %.1f and %.1f.\n"),
+			MIN_BRIGHTNESS, MAX_BRIGHTNESS);
 		exit(EXIT_FAILURE);
 	}
 
-	/* Color temperature at night */
-	if (temp_night < MIN_TEMP || temp_night >= MAX_TEMP) {
-		fprintf(stderr,
-			_("Temperature must be between %uK and %uK.\n"),
-			MIN_TEMP, MAX_TEMP);
-		exit(EXIT_FAILURE);
+	if (verbose) {
+		printf(_("Brightness: %.2f\n"), brightness);
 	}
 
 	/* Gamma */
@@ -981,7 +1051,7 @@ main(int argc, char *argv[])
 		if (verbose) printf(_("Color temperature: %uK\n"), temp);
 
 		/* Adjust temperature */
-		r = method->set_temperature(&state, temp, gamma);
+		r = method->set_temperature(&state, temp, brightness, gamma);
 		if (r < 0) {
 			fputs(_("Temperature adjustment failed.\n"), stderr);
 			method->free(&state);
@@ -989,10 +1059,24 @@ main(int argc, char *argv[])
 		}
 	}
 	break;
+	case PROGRAM_MODE_MANUAL:
+	{
+		if (verbose) printf(_("Color temperature: %uK\n"), temp_set);
+
+		/* Adjust temperature */
+		r = method->set_temperature(&state, temp_set, brightness, gamma);
+		if (r < 0) {
+			fputs(_("Temperature adjustment failed.\n"), stderr);
+			method->free(&state);
+			exit(EXIT_FAILURE);
+		}
+		
+	}
+	break;
 	case PROGRAM_MODE_RESET:
 	{
 		/* Reset screen */
-		r = method->set_temperature(&state, NEUTRAL_TEMP, gamma);
+		r = method->set_temperature(&state, NEUTRAL_TEMP, 1.0, gamma);
 		if (r < 0) {
 			fputs(_("Temperature adjustment failed.\n"), stderr);
 			method->free(&state);
@@ -1017,7 +1101,7 @@ main(int argc, char *argv[])
 		   will be exactly 6500K. */
 		float adjustment_alpha = 0.0;
 
-#ifdef HAVE_SYS_SIGNAL_H
+#ifdef HAVE_SIGNAL_H
 		struct sigaction sigact;
 		sigset_t sigset;
 		sigemptyset(&sigset);
@@ -1034,7 +1118,7 @@ main(int argc, char *argv[])
 		sigact.sa_mask = sigset;
 		sigact.sa_flags = 0;
 		sigaction(SIGUSR1, &sigact, NULL);
-#endif /* HAVE_SYS_SIGNAL_H */
+#endif /* HAVE_SIGNAL_H */
 
 		/* Continously adjust color temperature */
 		int done = 0;
@@ -1156,7 +1240,8 @@ main(int argc, char *argv[])
 			/* Adjust temperature */
 			if (!disabled || short_trans) {
 				r = method->set_temperature(&state,
-							    temp, gamma);
+							    temp, brightness,
+							    gamma);
 				if (r < 0) {
 					fputs(_("Temperature adjustment"
 						" failed.\n"), stderr);
